#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ToTarXz
#
# This class will essentially just create a .tar.xz package.
#
# The normal command for this is on Linux is:
#
#   tar cfJ <archive.tar.xz> <files>
#   tar cfJ rbt-04.09.2015.tar.xz rbt-04.09.2015/
#
# =========================================================================== #
# require 'totarxz/totarxz.rb'
# =========================================================================== #
require 'totarxz/requires/base_requires.rb'
require 'totarxz/version/version.rb'
require 'totarxz/constants.rb'

class ToTarXz # === Totarxz.new

  include Colours

  # ========================================================================= #
  # === initialize
  #
  # The first argument is the target to become a .tar.xz archive.
  # ========================================================================= #
  def initialize(
      package_this_target = nil,
      be_verbose          = BE_VERBOSE,
      run_already         = true,
      &block
    )
    @be_verbose = be_verbose # default
    reset # This must come after @be_verbose was set.
    be_verbose if be_verbose # Be silent not before calling sanitize_data().
    set_package_this_target(
      package_this_target
    )
    case run_already
    # ======================================================================= #
    # === --help
    # ======================================================================= #
    when /^-?-?help$/i
      show_help_then_exit
    # ======================================================================= #
    # === --date
    # ======================================================================= #
    when /^-?-?date$/,
         'TODAY','DATE','WITH_DATE'
      @repackage_archive_to_todays_date = true
      run_already = true
    end
    # ======================================================================= #
    # === Handle blocks next
    #
    # Intercept blocks here. Handle :be_verbose as argument, but also
    # certain keys in a Hash, since as of June 2020.
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      when :be_silent
        @be_verbose = false
      when :be_verbose
        @be_verbose = true
      else
        if yielded.is_a? Hash
          # ================================================================= #
          # === :use_this_archive_name => "htop-2.2.0"
          # ================================================================= #
          if yielded.has_key? :use_this_archive_name
            set_tar_xz_name(yielded.delete(:use_this_archive_name))
          end
        end
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    if @be_verbose
      @cmd = 'tar cJvf ' # v stands for verbosity here.
    else
      @cmd = 'tar cJf '  # v stands for verbosity here.
    end
    # ======================================================================= #
    # === @cmd
    # ======================================================================= #
    @cmd = @cmd.dup
    # ======================================================================= #
    # === @archive_type
    # ======================================================================= #
    @archive_type = '.tar.xz'
    # ======================================================================= #
    # === @repackage_archive_to_todays_date
    # ======================================================================= #
    @repackage_archive_to_todays_date = false
    # ======================================================================= #
    # === @orig_stdout
    # ======================================================================= #
    @orig_stdout = nil
  end

  # ========================================================================= #
  # === be_silent?
  # ========================================================================= #
  def be_silent?
    !@be_verbose
  end

  # ========================================================================= #
  # === be_verbose?
  # ========================================================================= #
  def be_verbose?
    @be_verbose
  end

  # ========================================================================= #
  # === show_help
  # ========================================================================= #
  def show_help
    e 'To package up with the date, use DATE or TODAY.'
  end

  # ========================================================================= #
  # === show_help_then_exit
  # ========================================================================= #
  def show_help_then_exit
    show_help
    exit
  end

  # ========================================================================= #
  # === abort_then_exit
  # ========================================================================= #
  def abort_then_exit
    abort "class ToTarXz: No file was provided or could be found. Please \n"\
          "supply a valid argument (an existing file) and try again."
    exit
  end

  # ========================================================================= #
  # === set_package_this_target
  #
  # This method will be called from within initialize().
  # ========================================================================= #
  def set_package_this_target(
      i = nil
    )
    if i.is_a? Array and !i.empty?
      i = i.join(' ').strip
    end
    if i and i.end_with?('*')
      # In this case continue.
    elsif i.nil? or !File.exist?(i.to_s)
      abort_then_exit
    end
    case i
    # ======================================================================= #
    # === --help
    # ======================================================================= #
    when /^-?-?help$/i
      show_help_then_exit
    end 
    i = i.to_s.dup
    unless i.end_with? '*'
      i << '/' unless i.end_with? '/'
    end
    i.gsub!(/\.tar\.xz$/, '')  if i.include? '.tar.xz'
    i.gsub!(/\.tar\.bz2$/, '') if i.include? '.tar.bt2'
    @package_this_target = i
    # ======================================================================= #
    # Now we can determine the .tar.xz name.
    # ======================================================================= #
    determine_tar_xz_name
  end; alias set_archive_filename       set_package_this_target # === set_archive_filename
       alias directory=                 set_package_this_target # === directory=
       alias extract_to_this_directory= set_package_this_target # === extract_to_this_directory=
  
  # ========================================================================= #
  # === silence
  # ========================================================================= #
  def silence
    @be_verbose = false
    @orig_stdout = $stdout # Be silent.
    $stdout = File.new('/dev/null', 'w')
  end

  # ========================================================================= #
  # === extract_to_this_directory?
  # ========================================================================= #
  def extract_to_this_directory?
    @tar_xz_name
  end; alias directory?        extract_to_this_directory? # === directory?
       alias dir?              extract_to_this_directory? # === dir?
       alias tar_xz_name?      extract_to_this_directory? # === tar_xz_name?
       alias archive_location? extract_to_this_directory? # === archive_location?

  # ========================================================================= #
  # === location?
  # ========================================================================= #
  def location?
    determine_tar_xz_name # Yes, we will determine anew.
  end
  
  # ========================================================================= #
  # === package_this_target?
  # ========================================================================= #
  def package_this_target?
    @package_this_target
  end; alias input?            package_this_target? # === input?
       alias archive_filename? package_this_target? # === archive_filename?
       alias archive?          package_this_target? # === archive?

  # ========================================================================= #
  # === mv
  # ========================================================================= #
  def mv(old, new)
    FileUtils.mv(old, new)
  end

  # ========================================================================= #
  # === today?
  # ========================================================================= #
  def today?
    Time.now.strftime '%d.%m.%Y'
  end

  # ========================================================================= #
  # === set_cmd
  # ========================================================================= #
  def set_cmd(i)
    unless i.end_with? ' '
      i = i.dup if i.frozen?
      i << ' '
    end
    @cmd = i
  end

  # ========================================================================= #
  # === run_cmd
  # ========================================================================= #
  def run_cmd
    e @cmd
    system @cmd
    # ======================================================================= #
    # Next, we will apply a mv-action if we repackage to another name.
    # ======================================================================= #
    if rename?
      new_name = File.basename(archive?).sub(/\.tar\.xz$/, '')+'-'+
                 today?+@archive_type
      opn; e 'Next renaming the archive to `'+sfancy(new_name)+'`.'
      mv(archive?, new_name)
    end
    $stdout = @orig_stdout unless be_verbose? # Restore stdout again.
  end; alias create_tarxz_package run_cmd # === create_tarxz

  # ========================================================================= #
  # === create_tarxz_package
  # ========================================================================= #
  def create_tarxz_package
    build_cmd_string
    run_cmd
  end

  # ========================================================================= #
  # === rename?
  #
  # Whether we will rename our archive or whether we will not.
  # ========================================================================= #
  def rename?
    @repackage_archive_to_todays_date
  end

  # ========================================================================= #
  # === determine_tar_xz_name
  #
  # Note that this method will always append .tar.xz to the name. This
  # is fine for .tar.xz archives but not for .tar.bz2 archives.
  # ========================================================================= #
  def determine_tar_xz_name(
      i = input?.delete('/')+
          TAR_XZ_EXTENSION
    )
    @tar_xz_name = i
  end

  # ========================================================================= #
  # === set_tar_xz_name
  # ========================================================================= #
  def set_tar_xz_name(i)
    if i.frozen?
      i = i.dup
    end
    i << TAR_XZ_EXTENSION unless i.end_with? TAR_XZ_EXTENSION
    @tar_xz_name = i
  end

  # ========================================================================= #
  # === build_cmd_string
  # ========================================================================= #
  def build_cmd_string
    @cmd << @tar_xz_name << ' ' << input? # Simply append.
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    create_tarxz_package
  end

  # ========================================================================= #
  # === ToTarXz[]
  # ========================================================================= #
  def self.[](i)
    new(i)
  end

end

if __FILE__ == $PROGRAM_NAME
  require 'opn'
  require 'colours/colours_e_autoinclude.rb'
  if ARGV[1]
    if ARGV[1].include?('TODAY') or ARGV[1].include?('DATE') # Handle TODAY and DATE differently.
      ToTarXz.new(ARGV.first, ARGV[1])
    end
  else
    ARGV.each {|entry|
      _ = ToTarXz.new(entry) { :be_silent }
      Opn.opn(namespace: ToTarXz.to_s)
      puts 'You can now find everything at: '+simp(_.archive_location?)
    }
  end
end # tottarxz GEAS TODAY